import lt from "long-timeout";
import sql from "../../../inc/sql";
import { clients } from "../../wrapper";

export default new class timer {
  constructor() {
    this.regex = /(\d+)(mon|[smhdwy])/;
    this.calc = {
      y:    val => val * 365 * 24 * 60 * 60,  // years
      mon:  val => val * 30 * 24 * 60 * 60,   // months
      w:    val => val * 7 * 24 * 60 * 60,    // weeks
      d:    val => val * 24 * 60 * 60,        // days
      h:    val => val * 60 * 60,             // hours
      m:    val => val * 60,                  // minutes
      s:    val => val                        // seconds
    };
    this._timers = new Set();
    setTimeout(() => { this.getTimers(); }, 1000);
  }
  add(time, fn, begin = ~~(Date.now()/1000)) {
    return new Promise((resolve, reject) => {
      if(this.regex.test(time)) {
        let seconds = 0;
        time.match(/\d+(mon|[smhdwy])/g).forEach(t => {
          const [,val,mod] = t.match(/(\d+)(mon|[smhdwy])/);
          seconds += parseInt(this.calc[mod](val));
        });
        if(seconds > 0) {
          const rest = seconds - (~~(Date.now() / 1000) - begin);
          lt.setTimeout(() => {
            fn();
          }, rest * 1000);
          this._timers.add(rest);
          resolve(rest);
        }
        else
          reject();
      }
      else
        reject();
    });
  }
  getTimers() {
    sql.any("select * from nxy_timers")
      .then(rows => {
        if(rows.length === 0)
          return;
        rows.forEach(r => {
          r.target = JSON.parse(r.target);
          clients.forEach(client => {
            if(client.type.toLowerCase() === r.target.type.toLowerCase()
            && client.client.network.toLowerCase() === r.target.network.toLowerCase())
            {
              this.add(r.delay, () => {
                client.client.sendmsg("normal", r.target.channel, client.client.format(`[b]${r.target.nick}[/b]: ${r.message} [i](${r.delay})[/i]`));
              }, r.created).catch(err => {});
            }
          });
        });
      }).catch(err => console.log(err));
  }
}
